/*
 * Copyright (C) 2014 Yahoo Inc.
 */

// A constant prefix which is part of every attachment id
var ATTACHMENT_ID_PREFIX = "ymail_attachmentId";

//A constant prefix which is part of every dropbox attachment id
var DROPBOX_ATTACHMENT_ID_PREFIX = "ymail_dropboxAttachmentId";

// A place holder token representing the attachment image data or the image url
var ATTACHMENT_URL_TOKEN = "#DATA_URL#";

// A place holder token representing the attachment name
var ATTACHMENT_NAME_TOKEN = "#ATT_NAME#";

// Each image attachment is represent by this template
var IMAGE_ATTACHMENT_TEMPLATE = "<img src=\"./images/attachments_delete_button.png\" class=\"attachment-delete-btn\"><img src=\"" + ATTACHMENT_URL_TOKEN + "\" class=\"attachment-image\">";

// Each non-image attachment is represent by this template
var NON_IMAGE_ATTACHMENT_TEMPLATE = IMAGE_ATTACHMENT_TEMPLATE + "<span class=\"non-image-attachment-label default-text-color\">" + ATTACHMENT_NAME_TOKEN + "</span>";

/**
 * An object which controls the attachment band, including adding attachments into view and showing/hiding the detailed view.
 */
var attachmentsBandController = {
	attachmentsContainerNode: null,
	attachmentsLabelNode: null,
	attachmentsSpinnerIconNode: null,
	attachmentsPreviewContainerNode: null,
	attachmentsNode: null,
	dropboxAttachmentsNode: null,
	
	isPreviewContainerOpen: false,
	numAttachments: 0,
	numDropboxAttachments: 0,
	totalSizeBytes: 0,
	
	/**
	 * A method which starts the attachmentsBandController 
	 */
	init: function(attachmentsContainerNode, attachmentsLabelNode, attachmentsSpinnerIconNode, attachmentsPreviewContainerNode, attachmentsNode, dropboxAttachmentsNode) {
		this.attachmentsContainerNode = attachmentsContainerNode;
		this.attachmentsLabelNode = attachmentsLabelNode;
		this.attachmentsSpinnerIconNode = attachmentsSpinnerIconNode;
		this.attachmentsPreviewContainerNode = attachmentsPreviewContainerNode;
		this.attachmentsNode = attachmentsNode;
		this.dropboxAttachmentsNode = dropboxAttachmentsNode;
		
		this.attachmentsLabelNode.addEventListener("click", this.toggleAttachmentsBand.bind(this), false);
		this.attachmentsSpinnerIconNode.addEventListener("click", this.toggleAttachmentsBand.bind(this), false);
	},
	
	/**
	 * Add an attachment with the given parameters. Usually called by native code.
	 */
	addAttachment: function(id, name, size, imageUri, shouldExpand, isTypeImage) {
		var attachmentNode = document.getElementById(ATTACHMENT_ID_PREFIX + id);
		
		// Attachment with the given id is already present, hence update the existing node
		if (attachmentNode) {
			// Update the image
			if (!imageUri) {
				imageUri = isTypeImage ? "./images/attachments_img_no_preview.png" : "./images/attachments_generic.png";
			}
			
			if (isTypeImage) {
				attachmentNode.innerHTML = IMAGE_ATTACHMENT_TEMPLATE.replace(ATTACHMENT_URL_TOKEN, imageUri);
				
				// Add an image load error handler.
				//
				// We need to do this in a setTimeout, since we need to wait for the DOM to update before we open the band
				// JavaScript executes in a single threaded environment, thus DOM mutation events such as above are posted on
				// this single thread. By doing a setTimeout(), we make sure that we schedule the method after the DOM mutation
				// event has had happened. Thus, the actual timeout value has no significance.
				setTimeout(function() {
					attachmentNode.getElementsByClassName("attachment-image")[0].addEventListener("error", this.attachmentImageLoadErrorHandler.bind(this), false);
				}.bind(this), 1);
			} else {
				attachmentNode.innerHTML = NON_IMAGE_ATTACHMENT_TEMPLATE.replace(ATTACHMENT_URL_TOKEN, imageUri).replace(ATTACHMENT_NAME_TOKEN, name);
			}
			
			// Update the size
			attachmentNode.setAttribute("attachment-size", size);
			this.totalSizeBytes -= parseInt(attachmentNode.getAttribute("attachment-size"), 10);
			this.totalSizeBytes += size;
			
			this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments + this.numDropboxAttachments, this.totalSizeBytes);
		} else {
			// New attachment, so create a node and configure it
			attachmentNode = document.createElement("span");
			attachmentNode.setAttribute("class", "attachment");
			attachmentNode.setAttribute("id", ATTACHMENT_ID_PREFIX + id);
			attachmentNode.setAttribute("attachment-size", size);

			if (!imageUri) {
				isTypeImage = false;
				imageUri = isTypeImage ? "./images/attachments_img_no_preview.png" : "./images/attachments_generic.png";
			}
			
			if (isTypeImage) {
				attachmentNode.innerHTML = IMAGE_ATTACHMENT_TEMPLATE.replace(ATTACHMENT_URL_TOKEN, imageUri);
				
				// Add an image load error handler.
				//
				// We need to do this in a setTimeout, since we need to wait for the DOM to update before we open the band
				// JavaScript executes in a single threaded environment, thus DOM mutation events such as above are posted on
				// this single thread. By doing a setTimeout(), we make sure that we schedule the method after the DOM mutation
				// event has had happened. Thus, the actual timeout value has no significance.
				setTimeout(function() {
					attachmentNode.getElementsByClassName("attachment-image")[0].addEventListener("error", this.attachmentImageLoadErrorHandler.bind(this), false);
				}.bind(this), 1);
			} else {
				attachmentNode.innerHTML = NON_IMAGE_ATTACHMENT_TEMPLATE.replace(ATTACHMENT_URL_TOKEN, imageUri).replace(ATTACHMENT_NAME_TOKEN, name);
			}
			
			attachmentNode.addEventListener("click", this.attachmentClickHandler.bind(this), false);
			
			this.numAttachments++;
			this.totalSizeBytes += size;
			
			if (this.numAttachments === 1) {
				this.attachmentsNode.classList.remove("hidden");
				this.attachmentsContainerNode.classList.remove("hidden");
			}
			
			this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments + this.numDropboxAttachments, this.totalSizeBytes);
			this.attachmentsNode.appendChild(attachmentNode);
		}
		
		if (shouldExpand) {
			// We need to do this in a setTimeout, since we need to wait for the DOM to update before we open the band
			// JavaScript executes in a single threaded environment, thus DOM mutation events such as above are posted on
			// this single thread. By doing a setTimeout(), we make sure that we schedule the method after the DOM mutation
			// event has had happened. Thus, the actual timeout value has no significance.
			setTimeout(this.expandAttachmentsBand.bind(this), 1);
		}
	},
	
	/**
	 * Adds a dropbox attachment with the given parameters. Usually called by native code.
	 */
	addDropboxAttachment: function(id, name, imageUri, shareUrl, shouldExpand) {
		var dropboxAttachmentNode = document.getElementById(DROPBOX_ATTACHMENT_ID_PREFIX + id);
		
		// Attachment with the given id is already present, hence update the existing node
		if (dropboxAttachmentNode) {
			dropboxAttachmentNode.setAttribute("share-url", shareUrl);
			dropboxAttachmentNode.innerHTML = NON_IMAGE_ATTACHMENT_TEMPLATE.replace(ATTACHMENT_URL_TOKEN, imageUri ? imageUri : "./images/attachments_generic.png").replace(ATTACHMENT_NAME_TOKEN, name);
			
			this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments + this.numDropboxAttachments, this.totalSizeBytes);
		} else {
			// New attachment, so create a node and configure it
			dropboxAttachmentNode = document.createElement("span");
			dropboxAttachmentNode.setAttribute("class", "attachment");
			dropboxAttachmentNode.setAttribute("id", DROPBOX_ATTACHMENT_ID_PREFIX + id);
			dropboxAttachmentNode.setAttribute("file-name", name);
			dropboxAttachmentNode.setAttribute("share-url", shareUrl);
			
			dropboxAttachmentNode.innerHTML = NON_IMAGE_ATTACHMENT_TEMPLATE.replace(ATTACHMENT_URL_TOKEN, imageUri ? imageUri : "./images/attachments_generic.png").replace(ATTACHMENT_NAME_TOKEN, name);
			
			dropboxAttachmentNode.addEventListener("click", this.dropboxAttachmentClickHandler.bind(this), false);
			
			this.numDropboxAttachments++;
			
			if (this.numDropboxAttachments === 1) {
				this.dropboxAttachmentsNode.classList.remove("hidden");
				this.attachmentsContainerNode.classList.remove("hidden");
			}
			
			this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments + this.numDropboxAttachments, this.totalSizeBytes);
			this.dropboxAttachmentsNode.appendChild(dropboxAttachmentNode);
		}
		
		if (shouldExpand) {
			// We need to do this in a setTimeout, since we need to wait for the DOM to update before we open the band
			// JavaScript executes in a single threaded environment, thus DOM mutation events such as above are posted on
			// this single thread. By doing a setTimeout(), we make sure that we schedule the method after the DOM mutation
			// event has had happened. Thus, the actual timeout value has no significance.
			setTimeout(this.expandAttachmentsBand.bind(this), 1);
		}
	},
	
	/**
	 * Remove an attachment with the given id
	 */
	removeAttachment: function(id) {
		var attachmentNode = document.getElementById(ATTACHMENT_ID_PREFIX + id);
		
		if (attachmentNode) {
			attachmentNode.parentNode.removeChild(attachmentNode);
			
			this.numAttachments--;
			
			this.totalSizeBytes -= parseInt(attachmentNode.getAttribute("attachment-size"), 10);
			this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments + this.numDropboxAttachments, this.totalSizeBytes);
			
			if (this.numAttachments === 0) {
				this.attachmentsNode.classList.add("hidden");
			}
			
			if (this.numAttachments + this.numDropboxAttachments === 0) {
				this.attachmentsContainerNode.classList.add("hidden");
			}
		}
	},
	
	/**
	 * Remove a dropbox attachment with the given id
	 */
	removeDropboxAttachment: function(id) {
		var dropboxAttachmentNode = document.getElementById(DROPBOX_ATTACHMENT_ID_PREFIX + id);
		
		if (dropboxAttachmentNode) {
			dropboxAttachmentNode.parentNode.removeChild(dropboxAttachmentNode);
			
			this.numDropboxAttachments--;
			
			this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments + this.numDropboxAttachments, this.totalSizeBytes);
			
			if (this.numDropboxAttachments === 0) {
				this.dropboxAttachmentsNode.classList.add("hidden");
			}
			
			if (this.numAttachments + this.numDropboxAttachments === 0) {
				this.attachmentsContainerNode.classList.add("hidden");
			}
		}
	},
	
	/**
	 * Expand the attachments band if it isn't expanded
	 */
	expandAttachmentsBand: function() {
		if (this.isPreviewContainerOpen === false) {
			this.toggleAttachmentsBand();
		}
	},
	
	/**
	 * Toggle the attachments band view by collapsing or expanding it
	 */
	toggleAttachmentsBand: function() {
		if (this.isPreviewContainerOpen === false) {
			this.attachmentsSpinnerIconNode.classList.remove("animation-rotate-0");
			this.attachmentsSpinnerIconNode.classList.add("animation-rotate-180");
			this.attachmentsPreviewContainerNode.classList.add("open");
			
			this.isPreviewContainerOpen = true;
		} else {
			this.attachmentsSpinnerIconNode.classList.remove("animation-rotate-180");
			this.attachmentsSpinnerIconNode.classList.add("animation-rotate-0");
			this.attachmentsPreviewContainerNode.classList.remove("open");
			
			this.isPreviewContainerOpen = false;
		}
		
		yMailBridge.setAttachmentsBandExpandState(this.isPreviewContainerOpen);
	},
	
	/**
	 * Click handler for each attachment
	 */
	attachmentClickHandler: function(event) {
		yMailBridge.showAttachmentClickOptions(event.currentTarget.id.replace(ATTACHMENT_ID_PREFIX, ""));
	},
	
	/**
	 * This method is used to handle errors while loading attachment preview images
	 */
	attachmentImageLoadErrorHandler: function(event) {
		event.target.src = "./images/attachments_img_no_preview.png";
	},
	
	/**
	 * Click handler for each dropbox attachment
	 */
	dropboxAttachmentClickHandler: function(event) {
		var dropboxAttachmentNode = event.currentTarget;
		yMailBridge.showDropboxAttachmentClickOptions(dropboxAttachmentNode.id.replace(DROPBOX_ATTACHMENT_ID_PREFIX, ""), 
				dropboxAttachmentNode.getAttribute("file-name"),
				dropboxAttachmentNode.getAttribute("share-url"));
	}
};