/*
 * Copyright (C) 2014 Yahoo Inc.
 */

// A constant which is used as the node attribute name for storing the previous hidden state
var CONTACT_SEARCH_ATTRIBUTE_NAME_LAST_STATE_HIDDEN = "data-last-state-is-hidden";

// A constant which is used as the node attribute name for storing the index of the suggestion node as per the suggestion list
var CONTACT_SEARCH_ATTRIBUTE_SMART_CONTACT_INDEX = "smart-contact-index";

// Url for the default profile image
var CONTACT_SEARCH_DEFAULT_PROFILE_IMAGE = "./images/default_profile_image.png";

// Delimiter characters for detecting 
var DELIMITER_CHAR_COMMA = ',';
var DELIMITER_CHAR_SEMICOLON = ';';

// The maximum height of the recipient row in pixels after which the lozenges will be collapsed
var COLLAPSE_LOZENGES_HEIGHT_THRESHOLD_PX = 150;

// A constant prefix which is part of every suggestion id
var CONTACT_SEARCH_ID_PREFIX_SUGGESTION = "ymail_contactSuggestion";

// A place holder token representing the profile image data or the image url
var CONTACT_SEARCH_PLACEHOLDER_IMAGE = "#CONTACTSEARCHIMAGE#";
	
// A place holder token representing the suggestion title
var CONTACT_SEARCH_PLACEHOLDER_TITLE = "#CONTACTSEARCHTITLE#";

// A place holder token representing the suggestion subtitle
var CONTACT_SEARCH_PLACEHOLDER_SUBTITLE = "#CONTACTSEARCHSUBTITLE#";

// The contact suggestion HTML template
var CONTACT_SEARCH_TEMPLATE_SUGGESTION = 
"<!-- Contact blacklist button -->" +
"<span class=\"right blacklist-button\">" +
"	<img src=\"./images/blacklist_contact_button.png\" class=\"blacklist-button-img\" />" +
"</span>" +
"" +	
"<!-- Suggestion profile image  -->" +
"<span class=\"left suggestion-profile-image-container\">" +
"	<img src=\"" + CONTACT_SEARCH_PLACEHOLDER_IMAGE + "\" class=\"suggestion-profile-img\" />" +
"</span>" +
"" +	
"<!-- Contact name -->" +
"<span class=\"center prevent-text-selection suggestion-name\">" +
"	<span class=\"prevent-text-selection suggestion-title\">" + CONTACT_SEARCH_PLACEHOLDER_TITLE + "</span>" +
"	<br/>" +
"	<span class=\"default-text-color prevent-text-selection\">" + CONTACT_SEARCH_PLACEHOLDER_SUBTITLE + "</span>" +
"</span>";

/**
 * An object which interacts with native code to provide type ahead search results
 */
var contactSearchController = {
	contactSearchManagers: [],
	isSearchModeOn: false,
	
	/**
	 * A method which creates instances of ContactSearchManager and holds onto the created object references
	 */
	create: function(recipientInputNode, searchSuggestionContainerNode, recipientRow, nodesToHideArray, uniqueTag) {
		if (!this.contactSearchManagers[uniqueTag]) {
			this.contactSearchManagers[uniqueTag] = new ContactSearchManager(recipientInputNode, searchSuggestionContainerNode, recipientRow, nodesToHideArray, uniqueTag);
		}
	},
	
	/**
	 * Shows the contact search results for the given recipient field. The results are given in form of a JSON Array.
	 * Stops the performance timer for the perfEvent (Online or Offline)
	 */
	showResults: function(resultsArrayJsonString, uniqueTag, perfEvent) {
		this.contactSearchManagers[uniqueTag].showResults(resultsArrayJsonString);
		yMailBridge.stopAutoCompleteTimer(perfEvent);
	},
	
	/**
	 * Removes a particular suggestion with the given id
	 */
	removeSuggestion: function(id, uniqueTag) {
		this.contactSearchManagers[uniqueTag].removeSuggestion(id);
	},
	
	/**
	 * Destroys the search suggestion mode, by clearing all results
	 */
	destroySearchMode: function(uniqueTag) {
		this.contactSearchManagers[uniqueTag].destroySearchMode();
	}
};

/**
 * Instances of this class manages type ahead search in a given recipient field
 */
function ContactSearchManager(recipientInputNode, searchSuggestionContainerNode, recipientRow, nodesToHideArray, uniqueTag) {
	this.recipientInputNode = recipientInputNode;
	this.searchSuggestionContainerNode = searchSuggestionContainerNode;
	this.recipientRow = recipientRow;
	this.nodesToHideArray = nodesToHideArray;
	this.areLozengesCollapsed = false;
	this.uniqueTag = uniqueTag;
	
	this.recipientInputNode.addEventListener("input", this.handleInput.bind(this), false);
}

/**
 * A method which handles user input and triggers the type ahead search
 */
ContactSearchManager.prototype.handleInput = function(event) {
	var typedText = event.target.value;
	yMailBridge.setRecipientTypedText(typedText, this.uniqueTag);
	
	if (typedText.length === 0) {
		// User has deleted all text in the input box, hence restore nodes to previous state
		this.destroySearchMode();
	} else {
		// User has typed something hence trigger a contact search
		yMailBridge.triggerContactSearch(typedText, this.uniqueTag);
	}
	
	// Check and turn typed text into lozenges
	var lastTypedChar = typedText[typedText.length-1];
	if (lastTypedChar === DELIMITER_CHAR_COMMA || lastTypedChar === DELIMITER_CHAR_SEMICOLON) {
		yMailBridge.addTypedTextAsContactToRecipientField(typedText.substr(0, typedText.length-1), this.uniqueTag);
	}
}

/**
 * A method which handles click events of individual suggestion
 */
ContactSearchManager.prototype.handleSuggestionClick = function(event) {
	var suggestionNode = event.currentTarget;
	
	var id = suggestionNode.id.replace(CONTACT_SEARCH_ID_PREFIX_SUGGESTION, "");
	var name = suggestionNode.getElementsByClassName("suggestion-title")[0].textContent;
	var isSmartContact = -1;
	
	if (suggestionNode.hasAttribute(CONTACT_SEARCH_ATTRIBUTE_SMART_CONTACT_INDEX)) {
		isSmartContact = parseInt(suggestionNode.getAttribute(CONTACT_SEARCH_ATTRIBUTE_SMART_CONTACT_INDEX));
	}
	
	yMailBridge.addSuggestionToRecipientField(id, name, isSmartContact, this.uniqueTag);
	
	this.recipientInputNode.value = "";
	this.recipientInputNode.focus();
	this.destroySearchMode();
}

/**
 * A method which handles click events of the black list contact button
 */
ContactSearchManager.prototype.handleBlacklistButtonClick = function(event) {
	var suggestionNode = event.target.parentNode.parentNode;
	if (suggestionNode) {
		yMailBridge.blacklistContact(suggestionNode.id.replace(CONTACT_SEARCH_ID_PREFIX_SUGGESTION, ""));
	}
	
	event.stopPropagation();
}

/**
 * Removes a particular suggestion with the given id
 */
ContactSearchManager.prototype.removeSuggestion = function(id) {
	var suggestionNode = document.getElementById(CONTACT_SEARCH_ID_PREFIX_SUGGESTION + id);
	
	if (suggestionNode) {
		if (suggestionNode.parentNode.childElementCount === 1) {
			this.destroySearchMode();
		} else {
			suggestionNode.parentNode.removeChild(suggestionNode);
		}
	}
}

/**
 * A method which sets up the UI in search mode, by hiding/showing certain elements
 */
ContactSearchManager.prototype.setupSearchMode = function() {
	if (!contactSearchController.isSearchModeOn) {
		contactSearchController.isSearchModeOn = true;
		
		// Hide everything in the nodes to hide array, as we need to show only the search results and the recipient input node
		for (var i=0; i<this.nodesToHideArray.length; i++) {
			var node = this.nodesToHideArray[i];
			
		    if (node.classList.contains("hidden")) {
		    	node.setAttribute(CONTACT_SEARCH_ATTRIBUTE_NAME_LAST_STATE_HIDDEN, "true");
		    } else {
		    	node.classList.add("hidden");
		    }
		}
		
		// If there are recipients already present as lozenges, the height of the recipient field might be really large
		// In such cases the suggestions list won't be visible any more, hence we collapse the lozenges in order to get more space
		if (this.recipientRow.clientHeight > COLLAPSE_LOZENGES_HEIGHT_THRESHOLD_PX) {
			this.areLozengesCollapsed = true;
			lozengeController.collapseLozenges(this.uniqueTag);
			this.searchSuggestionContainerNode.style.marginTop = (this.recipientRow.clientHeight-1) + "px";
		}
		
		this.recipientRow.classList.add("position-absolute-top");
		this.searchSuggestionContainerNode.style.marginTop = (this.recipientRow.clientHeight-1) + "px";
		this.searchSuggestionContainerNode.classList.remove("hidden");
	}
}

/**
 * A method which restores the UI from search mode into the previous state
 */
ContactSearchManager.prototype.destroySearchMode = function() {
	if (contactSearchController.isSearchModeOn) {
		contactSearchController.isSearchModeOn = false;
		
		for (var i=0; i<this.nodesToHideArray.length; i++) {
			var node = this.nodesToHideArray[i];
			
			if (node.hasAttribute(CONTACT_SEARCH_ATTRIBUTE_NAME_LAST_STATE_HIDDEN)) {
				node.removeAttribute(CONTACT_SEARCH_ATTRIBUTE_NAME_LAST_STATE_HIDDEN);
			} else {
				node.classList.remove("hidden");
			}
		}
		
		this.searchSuggestionContainerNode.classList.add("hidden");
		this.recipientRow.classList.remove("position-absolute-top");
		this.searchSuggestionContainerNode.style.marginTop = "";
		
		if (this.areLozengesCollapsed) {
			this.areLozengesCollapsed = false;
			lozengeController.expandLozenges(this.uniqueTag);
		}
		
		yMailBridge.setSearchModeOff();
	}
}

/**
 * A method which loads the default profile image, in case the profile image cannot be loaded from the server
 */
ContactSearchManager.prototype.handleProfileImageError = function(event) {
	event.target.src = CONTACT_SEARCH_DEFAULT_PROFILE_IMAGE;
}

/**
 * A method which adjusts the height and width of the profile image, so that the images are scaled according to aspect ratio
 */
ContactSearchManager.prototype.handleProfileImageLoad = function(event) {
	var imageNode = event.target;
	
	// We calculate the delta between the actual image dimension and the target dimension, which is the dimension of the parent container.
	// This delta allows us to figure out which dimension fits better, so that we can scale the other dimension automatically
	// E.g., Image Dimensions - 110w X 100h. Viewport - 50w X 50h. 
	// Deltas - 60w and 50h
	// Since the width delta is more, we fix the height to the height of the viewport and auto size the width
	var widthActualTargetDelta = imageNode.clientWidth - imageNode.parentNode.clientWidth;
	var heightActualTargetDelta = imageNode.clientHeight - imageNode.parentNode.clientHeight;
	
	if (widthActualTargetDelta > heightActualTargetDelta) {
		imageNode.style.width = "auto";
		imageNode.style.height = imageNode.parentNode.clientHeight + "px";
	} else {
		imageNode.style.width = imageNode.parentNode.clientWidth + "px";
		imageNode.style.height = "auto";
	}
}

/**
 * Clears all the suggestions in the suggestions container node by remove all children
 */
ContactSearchManager.prototype.clearSuggestions = function() {
	this.searchSuggestionContainerNode.classList.add("hidden");
	// Clear previous results
	while(this.searchSuggestionContainerNode.firstChild) {
		this.searchSuggestionContainerNode.removeChild(this.searchSuggestionContainerNode.firstChild);
	}
	this.searchSuggestionContainerNode.classList.remove("hidden");
}

/**
 * A method which accepts type ahead suggestion results in form a JSON Array String and shows them
 */
ContactSearchManager.prototype.showResults = function(resultsArrayJsonString) {
	if (resultsArrayJsonString && this.recipientInputNode.value) {
		try {
			var resultsArray = JSON.parse(resultsArrayJsonString);
			
			this.setupSearchMode();
			this.clearSuggestions();
			
			if (resultsArray.length === 0) {
				this.destroySearchMode();
			} else {
				for (var i=0; i<resultsArray.length; i++) {
					var result = resultsArray[i];
					var suggestionNode = document.createElement("div");
					suggestionNode.setAttribute("id", CONTACT_SEARCH_ID_PREFIX_SUGGESTION + result.id);
					suggestionNode.setAttribute("class", "header-row search-suggestion");
					suggestionNode.innerHTML = CONTACT_SEARCH_TEMPLATE_SUGGESTION
								.replace(CONTACT_SEARCH_PLACEHOLDER_IMAGE, result.image ? result.image : CONTACT_SEARCH_DEFAULT_PROFILE_IMAGE)
								.replace(CONTACT_SEARCH_PLACEHOLDER_TITLE, result.title)
								.replace(CONTACT_SEARCH_PLACEHOLDER_SUBTITLE, result.subtitle);
					
					var imageNode = suggestionNode.getElementsByClassName("suggestion-profile-img")[0];
					imageNode.addEventListener("error", this.handleProfileImageError.bind(this), false);
					imageNode.addEventListener("load", this.handleProfileImageLoad.bind(this), false);
					
					// If it isn't a smart contact then hide the blacklist button
					var blacklistButton = suggestionNode.getElementsByClassName("blacklist-button")[0];
					if (result.isSmart) {
						suggestionNode.setAttribute(CONTACT_SEARCH_ATTRIBUTE_SMART_CONTACT_INDEX, i);
						blacklistButton.addEventListener("click", this.handleBlacklistButtonClick.bind(this), true);
					} else {
						blacklistButton.classList.add("hidden");
					}
					
					suggestionNode.addEventListener("click", this.handleSuggestionClick.bind(this), false)
					
					this.searchSuggestionContainerNode.appendChild(suggestionNode);
				}
			}
		} catch(syntaxError) {
			this.destroySearchMode();
		}
	} else {
		this.destroySearchMode();
	}
}

