/*
 * Copyright (C) 2014 Yahoo Inc.
 */

/**
 * This file contains a set of functions which are called by native Java code and acts as the JavaScript interface exposed to the Java code
 */

/**
 * Gets the body from the native code and displays it
 */
function loadMessageBody() {
	document.getElementById(COMPOSE_CONTENT_ELEMENT_ID).innerHTML = yMailBridge.getContent();
}

/**
 * Toggles bold formatting
 */
function toggleBold() {
	document.execCommand('bold', false, null);
}

/**
 * Toggles italics formatting
 */
function toggleItalic() {
	document.execCommand('italic', false, null);
}

/**
 * Toggles underline formatting
 */
function toggleUnderline() {
	document.execCommand('underline', false, null);
}

/**
 * Sets the font size
 * @param size the size to the set in HTML <font> values [1-7]
 */
function setFontSize(size) {
	document.execCommand('fontSize', false, size);
}

/**
 * Sets the font color
 * @param color the color value in hex format - "#RRGGBB"
 */
function setFontColor(color) {
	document.execCommand('foreColor', false, color);
}

/**
 * Inserts a smiley at the current caret position
 * @param url the url of the smiley to be inserted
 */
function insertSmiley(url) {
	document.execCommand('insertHTML', false, " <img src='" + url + "'> ");
}

/**
 * Sets the from address
 * @param address the email address to be set
 */
function setFromAddress(address) {
	ccBccFromSummaryFieldController.setFromAddress(address);
}

/**
 * Sets the subject. Usually used after the activity is being restored from a configuration change such as device rotation.
 * @param subject the subject string
 */
function setSubject(subject) {
	subjectTextController.setSubject(subject);
}

/**
 * Sets the focus on the element with the given id. Usually used after being restored from a configuration change.
 * @param elementId the id of the element on which focus should be set.
 */
function setFocus(elementId) {
	var nodeToFocus = document.getElementById(elementId);
	if (nodeToFocus) {
		nodeToFocus.classList.remove("hidden");
		nodeToFocus.focus();
	}
}

/**
 * Sets the cursor at the position for elementId
 * @param elementId: the id of the element on which cursor should be set.
 * @param nodeText: text of the node.
 * @param position: position of the cursor in the node.
 */
function setBodyFocusAtPosition(elementId, nodeText, position) {
    composeContentFocusController.setBodyFocusAtPosition(elementId, nodeText, position);
}

/**
 * Sets the focus on the To recipient field
 */
function setFocusOnToRecipientField() {
	var toInputNode = document.getElementById(TO_INPUT_ELEMENT_ID);
	if (toInputNode) {
		// Make sure the lozenges are expanded otherwise the input field will be hidden
		lozengeController.expandLozenges(TAG_TO_RECIPIENT);
		toInputNode.focus();
	}
}

/**
 * Sets the focus on the body field
 */
function setFocusOnBody() {
	var bodyNode = document.getElementById(COMPOSE_CONTENT_ELEMENT_ID);
	if (bodyNode) {
		bodyNode.focus();
		
		// Scroll the window in order to ensure that the body is visible.
		setTimeout(function() {
			window.scrollBy(0, document.getElementById(COMPOSE_HEADER_ELEMENT_ID).clientHeight);
		}, 10);
	}
}

/**
 * Expands the summary field. Usually used after the activity is being restored from a configuration change such as device rotation.
 * @param setFocusFieldId an optional string parameter specifying the ID of the CC/BCC field onto which focus will be set after expanding the summary
 */
function expandSummaryField(setFocusFieldId) {
	setTimeout(ccBccFromSummaryFieldController.expandSummaryIntoDetailView.bind(ccBccFromSummaryFieldController, setFocusFieldId), 1);
}

/**
 * Adds an attachment to the attachments band
 * @param id the internal id of the attachment, which is passed back to the native code whenever the user manipulates any attachment
 * @param name the display name of the attachment
 * @param size the size in bytes of the attachment
 * @param image the image thumbnail url of the attachment. Can by local file:/// or a web url.
 * @param shouldExpand a boolean indicating whether the attachments band should be expanded after adding the attachment
 * @param isTypeImage a boolean indicating whether the attachment type is an image or not
 */
function addAttachment(id, name, size, imageUri, shouldExpand, isTypeImage) {
	setTimeout(attachmentsBandController.addAttachment.bind(attachmentsBandController, id, name, size, imageUri, shouldExpand, isTypeImage), 8);
}

/**
 * Adds a dropbox attachment to the attachments band
 * @param id the internal id of the dropbox attachment, which is passed back to the native code whenever the user manipulates any dropbox attachment
 * @param name the display name of the dropbox attachment
 * @param size the size in bytes of the dropbox attachment
 * @param image the image thumbnail url of the dropbox attachment. Can only be local file:/// url.
 * @param shareUrl the URL of the shared dropbox attachment. The attachment can be viewed using this URL. 
 * @param shouldExpand a boolean indicating whether the attachments band should be expanded after adding the dropbox attachment
 */
function addDropboxAttachment(id, name, imageUri, shareUrl, shouldExpand) {
	attachmentsBandController.addDropboxAttachment(id, name, imageUri, shareUrl, shouldExpand);
}

/**
 * Delete the attachment with the given id
 * @param id the id of the attachment to be deleted
 */
function deleteAttachment(id) {
	attachmentsBandController.removeAttachment(id);
}

/**
 * Delete the dropbox attachment with the given id
 * @param id the id of the dropbox attachment to be deleted
 */
function deleteDropboxAttachment(id) {
	attachmentsBandController.removeDropboxAttachment(id);
}

/**
 * Adds a lozenge with the given parameters
 * @param id the id of the lozenge to be added
 * @param name the name to be shown on the lozenge
 * @param shouldCollapse a boolean indicating whether the lozenges field should collapse after adding the given lozenge
 * @param uniqueTag the unique tag identifying a particular lozengeController
 */
function addContactLozenge(id, name, shouldCollapse, uniqueTag) {
	lozengeController.addLozenge(id, name, shouldCollapse, uniqueTag);
}

/**
 * Removes a lozenge with the given id
 * @param id the id of the lozenge to be removed
 * @param uniqueTag the unique tag identifying a particular lozengeController
 */
function removeContactLozenge(id, uniqueTag) {
	lozengeController.removeLozenge(id, uniqueTag);
}

/**
 * Clears the focused state of the lozenge with the given id
 * @param id the id of the lozenge whose focused state needs to be cleared
 * @param uniqueTag the unique tag identifying a particular lozengeController
 */
function clearLozengeFocusState(id, uniqueTag) {
	lozengeController.clearLozengeFocusState(id, uniqueTag);
}

/**
 * Sets the focused state of the lozenge with the given id
 * @param id the id of the lozenge whose focused state needs to be set
 * @param uniqueTag the unique tag identifying a particular lozengeController
 */
function setLozengeFocusState(id, uniqueTag) {
	lozengeController.setLozengeFocusState(id, uniqueTag);
}

/**
 * Sets the typed text in the given recipient field
 * 
 * @param text the text to be set
 * @param uniqueTag the unique tag identifying a particular lozengeController
 */
function setTypedText(text, uniqueTag) {
	lozengeController.setTypedText(text, uniqueTag);
}

/**
 * Shows the contact search results for the given recipient field. The results are given in form of a JSON Array. 
 * @param resultsJsonArray a JSON Array which encapsulates the results to be shown
 * @param uniqueTag the unique tag identifying a particular contactSearchController
 * @param perfEvent name for performance measurements. Either online (for xobni contacts) or offline (for device contacts).
 */
function showContactSearchResults(resultsJsonArray, uniqueTag, perfEvent) {
	setTimeout(contactSearchController.showResults.bind(contactSearchController, resultsJsonArray, uniqueTag, perfEvent), 10);
}

/**
 * Removes the suggestion with the given id
 * @param id the id of the suggestion to be removed
 * @param uniqueTag the unique tag identifying a particular contactSearchController
 */
function removeContactSuggestion(id, uniqueTag) {
	contactSearchController.removeSuggestion(id, uniqueTag);
}

/**
 * Clears all the contents of the recipient field and destroys any search suggestions being shown 
 * @param uniqueTag the unique tag identifying a particular contactSearchController
 */
function clearRecipientField(uniqueTag) {
	lozengeController.setTypedText("", uniqueTag);
	destroySearchMode(uniqueTag);
}

/**
 * Destroys any search suggestions being shown, but doesn't clear the recipient field
 * @param uniqueTag the unique tag identifying a particular contactSearchController
 */
function destroySearchMode(uniqueTag) {
	contactSearchController.destroySearchMode(uniqueTag);
}

/**
 * Invoked by Java to work around keyboard bugs on certain Android devices.
 * On these devices the delete key down events aren't generated correctly.
 * @see DeleteAndBackSpaceCapturingInputConnection.java for more info.
 */
function backSpaceKeyDownEvent() {
	var focusedElementId = yMailBridge.getCurrentlyFocusedElementId();
	if (focusedElementId) {
		var focusedElement = document.getElementById(focusedElementId);
		var event = document.createEvent("Event");
		event.initEvent("keydown", true, true);
		event.keyCode = KEYCODE_BACKSPACE;
		event.which = KEYCODE_BACKSPACE;
		
		if (focusedElementId.indexOf(TO_INPUT_ELEMENT_ID) >=0 || focusedElementId.indexOf(CC_INPUT_ELEMENT_ID) >=0 || focusedElementId.indexOf(BCC_INPUT_ELEMENT_ID) >=0) {
			focusedElement.dispatchEvent(event);
		}
	}
}

/**
 * Confirms the user choice of deleting the clicked image
 * @param imageId the image id passed by JavaScript of the image to be deleted.
 */
function confirmImageDelete(imageId) {
	imageTapDeleteController.confirmDelete(imageId);
}
